/*
 * This file is part of Phidget22.jar
 *
 * Copyright (c) 2017-2022 Phidgets Inc.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived from
 *    this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */

package com.phidget22;

public class Phidget extends PhidgetBase implements java.util.Comparator<Phidget> {
	private String _phidgetidstr;

	Phidget(long handle) {
		super(handle);
	}

	/**
	 * Return a Sring describing this Phidget.
	 */
	public String toString() {
		try {
			return (getDescription());
		}
		catch (PhidgetException ex) {
			return ("["+getClass().getName()+"]");
		}
	}

	public String getPhidgetIDString() {
		StringBuffer key;

		if (_phidgetidstr != null)
			return (_phidgetidstr);

		try {
			key = new StringBuffer();
			if (getIsLocal())
				key.append('L');
			else
				key.append('N');

			if (getIsChannel()) {
				key.append('_').append('C').append(getChannel());
				key.append('_').append(getChannelClass());
			} else {
				key.append('_').append('D').append('_').append(getDeviceClass());
				key.append('_').append(getDeviceID());
				if (getIsHubPortDevice())
					key.append('_').append('H');
			}

			key.append('_').append(getHubPort());
			key.append('_').append(getDeviceSerialNumber());

			_phidgetidstr = key.toString();
			return (_phidgetidstr);
		} catch (PhidgetException ex) {
			return ("_invalid_a_phidget_");
		}
	}

	public boolean equals(Object o) {

		if (o == null)
			return (false);

		if (!(o instanceof Phidget))
			return (false);

		return (hashCode() == o.hashCode());
	}

	public int hashCode() {

		return (getPhidgetIDString().hashCode());
	}

	public int compare(Phidget a, Phidget b) {

		try {
			/*
			 * Local is greater than remote.
			 */
			if (a.getIsLocal()) {
				if (b.getIsLocal() == false)
					return (1);
			} else {
				if (b.getIsLocal())
					return (-1);
			}

			int sndiff = a.getDeviceSerialNumber() - b.getDeviceSerialNumber();
			if (sndiff != 0)
				return (sndiff);

			int hpdiff = a.getHubPort() - b.getHubPort();
			if (hpdiff != 0)
				return (hpdiff);

			/*
			 * Channels are less than devices.
			 */
			if (a.getIsChannel()) {
				if (b.getIsChannel() == false)
					return (-1);

				int chdiff = a.getChannel() - b.getChannel();
				if (chdiff != 0)
					return (chdiff);

				int chcldiff = a.getChannelClassName().compareTo(b.getChannelClassName());
				if (chcldiff != 0)
					return (chcldiff);
			} else {
				if (b.getIsChannel())
					return (1);

				int dcldiff = a.getDeviceClassName().compareTo(b.getDeviceClassName());
				if (dcldiff != 0)
					return (dcldiff);

				int diddiff = a.getDeviceID().ordinal() - b.getDeviceID().ordinal();
				if (diddiff != 0)
					return (diddiff);

				/* Hub port devicse are less than */
				if (a.getIsHubPortDevice()) {
					if (!b.getIsHubPortDevice())
						return (-1);
				} else {
					if (b.getIsHubPortDevice())
						return (1);
				}
			}
		} catch (PhidgetException pe) {
			return (-1);
		}

		return (0);
	}

	protected void finalize() throws Throwable {

		try {
			deleteHandle();
			super.handle = 0;
		} finally {
			super.finalize();
		}
	}

	private native java.lang.String getDescription() throws PhidgetException;
	private native void deleteHandle() throws PhidgetException;
}
