/*
 * This file is part of Phidget22.jar
 *
 * Copyright (c) 2017-2022 Phidgets Inc.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived from
 *    this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */

package com.phidget22;

public class PhidgetException extends java.lang.Exception {
	private String description;
	private String detail;
	private ErrorCode errno;

	private PhidgetException(int errno, String description, String detail) {
		super("PhidgetException 0x" + String.format("%02x", errno) + " (" + description + ")\n" + detail);

		this.errno = ErrorCode.getEnum(errno);
		this.description = description;
		this.detail = detail;
	}

	private PhidgetException(int errno, String description) {
		super("PhidgetException 0x" + String.format("%02x", errno) + " (" + description + ")");

		this.errno = ErrorCode.getEnum(errno);
		this.description = description;
		this.detail = "";
	}

	public ErrorCode getErrorCode() {

		return (errno);
	}

	public String getDescription() {

		return (description);
	}

	public String getDetail() {

		return (detail);
	}

	public String toString() {

		if (detail.length() > 0)
			return ("PhidgetException 0x" + String.format("%02x", errno.getCode()) + " (" + description + ")\n" + detail);
		return ("PhidgetException 0x" + String.format("%02x", errno.getCode()) + " (" + description + ")");
	}
}
