/*
 * This file is part of libphidget22java
 *
 * Copyright (c) 2017-2022 Phidgets Inc.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived from
 *    this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "phidget_jni.h"
#include "com_phidget22_DictionaryBase.h"

extern jfieldID phidget_handle_fid;

void com_phidget22_Dictionary_OnLoad(JNIEnv *env) { }

JNIEXPORT jstring JNICALL Java_com_phidget22_DictionaryBase_get(JNIEnv * env, jobject obj, jstring key) {
	PhidgetDictionaryHandle h = (PhidgetDictionaryHandle)(uintptr_t)(*env)->GetLongField(env, obj, phidget_handle_fid);
	int error;
	char value[2048];
	size_t len = 2048;
	const char *keyUTFChars = (key == NULL ? NULL : (*env)->GetStringUTFChars(env, key, NULL));
	if ((error = PhidgetDictionary_get(h, keyUTFChars, value, (size_t)len))) {
		if (error = EPHIDGET_NOENT)
			return NULL;
		PH_THROW_LAST_ERR(error);
		return NULL;
	}
	if (keyUTFChars != NULL) (*env)->ReleaseStringUTFChars(env, key, keyUTFChars);
	return (*env)->NewStringUTF(env, value);
}
