/*
 * This file is part of libphidget22java
 *
 * Copyright (c) 2017-2022 Phidgets Inc.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived from
 *    this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "phidget_jni.h"
#include "com_phidget22_Phidget.h"
#include "com_phidget22_PhidgetBase.h"

extern const char * CCONV channelInfo(PhidgetHandle, char *, uint32_t);
extern const char * CCONV deviceInfo(PhidgetHandle, char *, uint32_t);

extern jfieldID phidget_handle_fid;
jclass phid_class;
jmethodID phid_cons;

void
com_phidget22_Phidget_OnLoad(JNIEnv *env) {

	if (!(phid_class = (*env)->FindClass(env, "com/phidget22/Phidget")))
		JNI_ABORT_STDERR("Coulnd't FindClass com/phidget22/Phidget");
	if (!(phid_class = (jclass)(*env)->NewGlobalRef(env, phid_class)))
		JNI_ABORT_STDERR("Couldn't create global ref phid_class");
	if (!(phid_cons = (*env)->GetMethodID(env, phid_class, "<init>", "(J)V")))
		JNI_ABORT_STDERR("Couldn't get Method ID <init> from phid_class");
}

JNIEXPORT jstring JNICALL
Java_com_phidget22_Phidget_getDescription(JNIEnv * env, jobject obj) {
	PhidgetHandle h = (PhidgetHandle)(uintptr_t)(*env)->GetLongField(env, obj, phidget_handle_fid);
	int channel;
	char buf[128];

	if (!h) {
		PH_THROW(EPHIDGET_UNEXPECTED);
		return NULL;
	}

	if (Phidget_getIsChannel(h, &channel)) {
		PH_THROW(EPHIDGET_UNEXPECTED);
		return NULL;
	}

	if (channel)
		return (*env)->NewStringUTF(env, channelInfo(h, buf, sizeof(buf)));
	return (*env)->NewStringUTF(env, deviceInfo(h, buf, sizeof(buf)));
}

JNIEXPORT void JNICALL
Java_com_phidget22_Phidget_deleteHandle(JNIEnv * env, jobject obj) {
	PhidgetHandle h;

	h = (PhidgetHandle)(uintptr_t)(*env)->GetLongField(env, obj, phidget_handle_fid);
	Phidget_delete(&h);
}

JNIEXPORT jobject JNICALL Java_com_phidget22_PhidgetBase_getParent(JNIEnv * env, jobject obj) {
	PhidgetHandle parent;
	PhidgetHandle h;
	int error;

	h = (PhidgetHandle)(uintptr_t)(*env)->GetLongField(env, obj, phidget_handle_fid);
	if ((error = Phidget_getParent(h, &parent))) {
		PH_THROW_LAST_ERR(error);
		return NULL;
	}

	if (parent == NULL)
		return (NULL);

	return ((*env)->NewObject(env, phid_class, phid_cons, parent));
}

JNIEXPORT jobject JNICALL Java_com_phidget22_PhidgetBase_getHub(JNIEnv * env, jobject obj) {
	PhidgetHandle hub;
	PhidgetHandle h;
	int error;

	h = (PhidgetHandle)(uintptr_t)(*env)->GetLongField(env, obj, phidget_handle_fid);
	if ((error = Phidget_getHub(h, &hub))) {
		PH_THROW_LAST_ERR(error);
		return NULL;
	}

	if (hub == NULL)
		return (NULL);
	return ((*env)->NewObject(env, phid_class, phid_cons, hub));
}